@@ -25,12 +25,13 @@ The CA connectors are written to the database table "caconnector".
 The code is tested in tests/test_api_caconnector.py.
 """
 from flask import (Blueprint, request)
-from .lib.utils import (send_result)
+from .lib.utils import (send_result, getParam)
 from ..lib.log import log_with
 from flask import g
 import logging
 from privacyidea.lib.caconnector import (save_caconnector,
                                          delete_caconnector,
+                                         get_caconnector_specific_options,
                                          get_caconnector_list)
 from ..api.lib.prepolicy import prepolicy, check_base_action
 from privacyidea.lib.policy import ACTION
@@ -56,6 +57,21 @@ def get_caconnector_api(name=None):
     return send_result(res)
 
 
+@caconnector_blueprint.route('/specific/<catype>', methods=['GET'])
+@log_with(log)
+@prepolicy(check_base_action, request, ACTION.CACONNECTORREAD)
+def get_caconnector_specific(catype):
+    """
+    It requires the configuration data of a CA connector in the GET parameters
+    and returns a dict of possible specific options.
+    """
+    param = request.all_data
+    # Create an object out of the type and the given request parameters.
+    res = get_caconnector_specific_options(catype, param)
+    g.audit_object.log({"success": True})
+    return send_result(res)
+
+
 @caconnector_blueprint.route('/<name>', methods=['POST'])
 @log_with(log)
 @prepolicy(check_base_action, request, ACTION.CACONNECTORWRITE)
