@@ -206,19 +206,7 @@ myApp.controller("policyDetailsController", ["$scope", "$stateParams",
         $scope.resolversLoaded = true;
         check_all_loaded();
     });
-    $scope.params = {
-            action: "",
-            scope: "",
-            realm: "",
-            resolver: "",
-            user: "",
-            active: true,
-            check_all_resolvers: false,
-            client: "",
-            time: "",
-            priority: 1,
-            conditions: []
-        };
+
     $scope.existingPolicyname = $stateParams.policyname;
     if ($scope.existingPolicyname) {
         $scope.policyname = $scope.existingPolicyname;
@@ -307,6 +295,21 @@ myApp.controller("policyDetailsController", ["$scope", "$stateParams",
         }
     };
 
+    $scope.params = {
+        action: "",
+        scope: "",
+        realm: "",
+        resolver: "",
+        user: "",
+        active: true,
+        check_all_resolvers: false,
+        client: "",
+        time: "",
+        priority: 1,
+        conditions: [],
+        pinode: []
+    };
+
     $scope.createPolicy = function () {
         // This is called to save the policy
         // get scope
@@ -406,7 +409,6 @@ myApp.controller("policyDetailsController", ["$scope", "$stateParams",
             $scope.params.client = policy.client;
             $scope.params.time = policy.time;
             $scope.params.priority = policy.priority;
-            $scope.params.pinodes = policy.pinode;
             // we need to deep-copy the policy conditions to ensure that we're working on our own copy
             $scope.params.conditions = angular.copy(policy.conditions);
             // tick the realms and the resolvers
