@@ -20,7 +20,7 @@
  */
 
 //Return an empty array if string is empty.
-//Otherwise return the result of the ordinary split.
+//Otherwise, return the result of the ordinary split.
 String.prototype.mysplit = function(separator) {
     return this == "" ? [] : this.split(separator);
 };
@@ -34,7 +34,7 @@ angular.module("privacyideaApp")
                                    "hotkeys", "RegisterFactory",
                                    "U2fFactory", "webAuthnToken", "instanceUrl",
                                    "PollingAuthFactory", "$transitions",
-                                   "resourceNamePatterns",
+                                   "resourceNamePatterns", "$window",
                                    function (Idle, $scope, $sce, $http, $location,
                                              authUrl, validateUrl, AuthFactory, $rootScope,
                                              $state, ConfigFactory, inform,
@@ -42,7 +42,7 @@ angular.module("privacyideaApp")
                                              hotkeys, RegisterFactory,
                                              U2fFactory, webAuthnToken, instanceUrl,
                                              PollingAuthFactory, $transitions,
-                                             resourceNamePatterns) {
+                                             resourceNamePatterns, $window) {
 
     $scope.instanceUrl = instanceUrl;
     $scope.checkRight = AuthFactory.checkRight;
@@ -72,7 +72,7 @@ angular.module("privacyideaApp")
     obj = angular.element(document.querySelector('#LOGO'));
     $scope.piLogo = obj.val();
     obj = angular.element(document.querySelector('#HAS_JOB_QUEUE'));
-    $scope.hasJobQueue = obj.val() == "True";
+    $scope.hasJobQueue = obj.val() === "True";
     obj = angular.element(document.querySelector('#LOGIN_TEXT'));
     $scope.piLoginText = obj.val();
     obj = angular.element(document.querySelector('#SHOW_NODE'));
@@ -156,7 +156,7 @@ angular.module("privacyideaApp")
     });
 
     $scope.$on('IdleTimeout', function () {
-        if ($scope.timeout_action == "logout") {
+        if ($scope.timeout_action === "logout") {
             //debug: console.log("Logout!");
             $scope.logout();
         } else {
@@ -432,6 +432,7 @@ angular.module("privacyideaApp")
             if ($scope.qr_images.length > 0) {
                 $scope.qr_col_md = "col-md-" + parseInt(12 / $scope.qr_images.length);
             }
+            $scope.audit_page_size = data.result.value.audit_page_size;
             $scope.token_page_size = data.result.value.token_page_size;
             $scope.user_page_size = data.result.value.user_page_size;
             $scope.user_details_in_tokenlist = data.result.value.user_details;
@@ -443,6 +444,7 @@ angular.module("privacyideaApp")
             } else {
                 $scope.startRoute = "/token";
             }
+            $scope.logout_redirect_url = data.result.value.logout_redirect_url;
             $scope.hide_welcome = data.result.value.hide_welcome;
             $scope.hide_buttons = data.result.value.hide_buttons;
             $scope.show_seed = data.result.value.show_seed;
@@ -539,12 +541,17 @@ angular.module("privacyideaApp")
         Idle.unwatch();
         // Jump to top when the policy is saved
         $('html,body').scrollTop(0);
+        // Optional redirect on logout
+        if ($scope.logout_redirect_url !== "") {
+            console.log("Redirecting to "+$scope.logout_redirect_url)
+            $window.location.href = $scope.logout_redirect_url;
+        }
     };
 
     $scope.nextWelcome = function() {
         $scope.welcomeStep += 1;
-        if (($scope.subscription_state == 0 && $scope.welcomeStep === 4) ||
-            ($scope.subscription_state == 1 && $scope.welcomeStep === 5)) {
+        if (($scope.subscription_state === 0 && $scope.welcomeStep === 4) ||
+            ($scope.subscription_state === 1 && $scope.welcomeStep === 5)) {
             $('#dialogWelcome').modal("hide");
         }
     };
